{{
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// VGA64 Bitmap Engine
//
// Author: Kwabena W. Agyeman
// Updated: 7/15/2010
// Designed For: P8X32A
// Version: 1.1
//
// Copyright (c) 2010 Kwabena W. Agyeman
// See end of file for terms of use.
//
// Update History:
//
// v1.0 - Original release - 9/28/2009.
// v1.1 - Merged and rewrote code and added more features - 7/15/2010.
//
// For each included copy of this object only one spin interpreter should access it at a time.
//
// Nyamekye,
// modifications J Moxham April 2011 so can run from a cogject. 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


}}


DAT

' //////////////////////Variable Arrary////////////////////////////////////////////////////////////////////////////////////////

pixelColors             long    0                                                   ' Screen colors.
displayIndicator        byte    1                                                   ' Video output control.
syncIndicator           byte    0                                                   ' Video update control.
cogNumber               byte    0                                                   ' Cog ID.
bitsPerPixel            byte    0                                                   ' Bits ID.

PUB plotPixel(pixelValue, xPixel, yPixel, screen) '' 7 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Plots a 1x1 pixel on screen.
'' //
'' // PixelValue - The pixel to plot on screen. Between %%0 and %%1 or between %%0, %%1, %%2, and %%3 depending on color mode.
'' // XPixel - The X cartesian pixel coordinate, will be forced to be multiple of 1. Y will be forced to be a multiple of 1.
'' // YPixel - The Y cartesian pixel coordinate. Note that this axis is inverted like on all other graphics drivers.
'' // DisplayBase - The address of the display buffer to draw to.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  xPixel := ((xPixel <# (320 - 1)) #> 0)  ' 320 = horizontal pixels
  screen += (((20 * ((yPixel <# (240 - 1)) #> 0)) + (xPixel >> (5 - bitsPerPixel))) << 2) ' 20 is horizontal longs, 240 = vertical pixels

  xPixel := ((xPixel & ($1F >> bitsPerPixel)) << bitsPerPixel)
  yPixel := (!((1 + (bitsPerPixel << 1)) << xPixel))

  long[screen] := ((long[screen] & yPixel) | (((pixelValue <# (1 + (bitsPerPixel << 1))) #> 0) << xPixel))

PUB displayState(state) '' 4 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Enables or disables the BMP Driver's video output - turning the monitor off or putting it into standby mode.
'' //
'' // State - True for active and false for inactive.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  displayIndicator := state

PUB displayRate(rate) '' 4 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Returns true or false depending on the time elasped according to a specified rate.
'' //
'' // Rate - A display rate to return at. 0=0.234375Hz, 1=0.46875Hz, 2=0.9375Hz, 3=1.875Hz, 4=3.75Hz, 5=7.5Hz, 6=15Hz, 7=30Hz.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  result or= (($80 >> ((rate <# 7) #> 0)) & syncIndicator)

PUB displayWait(frames) '' 4 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Waits for the display vertical refresh.
'' //
'' // The best time to draw on screen for flicker free operation is right after this function returns.
'' //
'' // Frames - Number of vertical refresh frames to wait for.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  repeat (frames #> 0)
    result := syncIndicator
    repeat until(result <> syncIndicator)

PUB displayColor(pixelNumber, newColor) '' 5 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Changes a pixel color for the whole screen.
'' //
'' // PixelNumber - The pixel number to change for the whole screen. Between 0 and 1 or 0 to 3.
'' // NewColor - A color byte (%RR_GG_BB_xx) describing the pixel's new color for the whole screen.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  pixelColors.byte[(pixelNumber <# (1 + (bitsPerPixel << 1))) #> 0] := newColor

PUB displayClear(patternValue, displayBase) '' 5 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Clears the whole screen.
'' //
'' // PatternValue - The pattern to plot on screen.
'' // DisplayBase - The address of the display buffer to draw to.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  longfill(displayBase, patternValue, (20 * 240))    ' 20 is horizontal longs, 240 is vertical pixels

PUB BMPEngineStart(cogarray, screen, pinGroup, colorMode, horizontalResolution, verticalResolution) '' 11 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Starts up the BMP driver running on a cog.
'' //
'' // Returns true on success and false on failure.
'' //
'' // PinGroup - Pin group to use to drive the video circuit. Between 0 and 3.
'' // ColorMode - Color mode to use for the whole screen. Between 1 bit per pixel or 2 bits per pixel.
'' // HorizontalResolution - The driver will force this value to be a factor of 640 and divisible by 16 or 32. 16/32 to 640.
'' // VerticalResolution - The driver will force this value to be a factor of 480. 1 to 480.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    long[screen][0] := screen                           ' pointer to screen buffer
    long[screen][1] := displayindicator                 'pass variables to cog via screen buffer
    long[screen][2] := syncindicator
    pinGroup := ((pinGroup <# 3) #> 0)
    bitsPerPixel := ((colorMode <# 2) #> 1)
    long[screen][3] := ($FF << (8 * pinGroup))          ' directionstate
    long[screen][4] := ($20_00_00_FF | (pinGroup << 9) | ((--bitsPerPixel) << 28)) ' videostate   
    pinGroup := constant((25_175_000 + 1_600) / 4)
    long[screen][5] := 1                                ' frequencystate                                    
    repeat 32
      pinGroup <<= 1
      long[screen][5] <-= 1
      if(pinGroup => clkfreq)
        pinGroup -= clkfreq
        long[screen][5] +=1      

    horizontalResolution := ((horizontalResolution <# 640) #> (32 >> bitsPerPixel))
    repeat while((640 // horizontalResolution) or ((horizontalResolution--) // (32 >> bitsPerPixel)))
    long[screen][6] := (640 / (++horizontalResolution))  ' 6 = horizontalscaling

    verticalResolution := ((verticalResolution <# 480) #> 1)
    repeat while(480 // verticalResolution--)
    long[screen][7] := (480 / (++verticalResolution))     ' 7 = verticalscaling
    long[screen][8]  := (640 / long[screen][6])           ' 8 = horizontalpixels
    long[screen][9] :=  (480 / long[screen][7])           ' 9 = verticalpixels
    long[screen][10] := ((long[screen][6] << 12) + ((constant(640 * 32) >> bitsPerPixel) / long[screen][8]))  ' 10 is visible scale
    long[screen][11] := (((8 << bitsPerPixel) << 12) + 160) ' 11 is invisible scale
    long[screen][12] := (long[screen][8] / (32 >> bitsPerPixel))  ' 12 is horizontal longs (equals 20 for 320x240)
    long[screen][13] := (long[screen][12] * 4)                   ' 13 is horizontalloops
    long[screen][14] := @pixelColors                             ' 14 is pixel colors

    'cogNumber := cognew(@initialization, screen)
    cognumber := cognew(cogarray,screen) ' start cog 
    waitcnt((((clkfreq / 1000 * 1) - 4296) #> 381) + cnt)  ' 1msec delay to read in values to cog
    result or= ++cogNumber

PUB BMPEngineStop '' 3 Stack Longs

'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
'' // Shuts down the BMP driver running on a cog.
'' ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

  if(cogNumber)
    cogstop(-1 + cogNumber~)
{{

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                  TERMS OF USE: MIT License
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the
// Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
// COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
}}